angular.module('tuinvogels.controllers-gids', [])

/***************************/
/* VogelsController        */
/***************************/
.controller('VogelsController', function($scope, $state, $ionicModal, $ionicScrollDelegate, $cordovaGoogleAnalytics, DataService, FilterFactory) {

    try {
        $cordovaGoogleAnalytics.trackView('Vogelgids');
    } catch (e) {};

    $scope.search = {};
    $scope.colors = DataService.colors;
    //$scope.beaks = DataService.beaks;
    //$scope.silhouettes = DataService.silhouettes;
    $scope.filter = FilterFactory.getFilter();

    $scope.changeOptions = {
        1: 'Zeer kleine trefkans',
        2: 'Kleine trefkans',
        3: 'Redelijke trefkans',
        4: 'Grote trefkans'
    }

    var initDisplayItem = function(items, filterarray) {
        var item = {},
            found;

        for (var i = 0; i < items.length; i++) {
            found = filterarray.indexOf(items[i].id) > -1;
            item[items[i].id] = found;
        }
        return item;
    };
    $scope.colorDisplay = initDisplayItem($scope.colors, $scope.filter.colors);
    //$scope.silhouetteDisplay = initDisplayItem($scope.silhouettes, $scope.filter.silhouettes);
    //$scope.beakDisplay = initDisplayItem($scope.beaks, $scope.filter.beaks);


    $scope.sizeItems = $scope.filter.size ? 1 : 0;
    $scope.colorItems = $scope.filter.colors.length;
    //$scope.silhouetteItems = $scope.filter.silhouettes.length;
    //$scope.beakItems = $scope.filter.beaks.length;
    $scope.showReset = $scope.colorItems > 0 || $scope.sizeItems > 0;
    $scope.numFilter = $scope.colorItems + $scope.sizeItems;

    $scope.birds = FilterFactory.determin(_.filter(DataService.birds, {active: 1}), $scope.filter);   
    $scope.activeBirds = $scope.birds.length;


    $scope.scrollTop = function() {
        $ionicScrollDelegate.scrollTop();
    };

    $scope.clearSearch = function() {
        $scope.search.bird = "";
    };

    $ionicModal.fromTemplateUrl('templates/filter-modal.html', {
        scope: $scope,
        animation: 'slide-in-right'
    }).then(function(modal) {
        $scope.modal = modal;
    });

    $scope.goState = function(state) {
        $state.go(state);
    }

    $scope.showDetails = function(id) {
        FilterFactory.setCurrentBirds(_.pluck($scope.birds, 'id'));
        $state.go('app.vogels-details', {
            id: id
        });
    }

    $scope.resetFilter = function() {
        $scope.filter = FilterFactory.resetFilter();

        $scope.showReset = false;
        $scope.search.bird = "";
        $scope.sizeItems = 0;

        $scope.colorItems = 0;
        //$scope.silhouetteItems = 0;
        //$scope.beakItems = 0;
        $scope.numFilter = 0;

        $scope.colorDisplay = initDisplayItem($scope.colors, $scope.filter.colors);
        //$scope.silhouetteDisplay = initDisplayItem($scope.silhouettes, $scope.filter.silhouettes);
        //$scope.beakDisplay = initDisplayItem($scope.beaks, $scope.filter.beaks);

        $scope.birds = FilterFactory.determin(_.filter(DataService.birds, {active: 1}), $scope.filter);  
        $scope.scrollTop();
    }

    $scope.onToggleFeature = function() {
        FilterFactory.setFilter($scope.filter);

        $scope.showReset = $scope.colorItems > 0 || $scope.sizeItems > 0;
        $scope.numFilter = $scope.colorItems + $scope.sizeItems;

        $scope.birds = FilterFactory.determin(DataService.birds);
        $scope.scrollTop();
    }

    $scope.setSize = function(size) {
        if ($scope.filter.size && $scope.filter.size == size) $scope.filter.size = null;
        else $scope.filter.size = size;

        $scope.birds = FilterFactory.determin(DataService.birds);
        $scope.sizeItems = $scope.filter.size ? 1 : 0;
        $scope.onToggleFeature();
    }

    $scope.toggleColor = function(colorid) {
        toggleArray($scope.filter.colors, colorid);

        $scope.colorDisplay[colorid] = !$scope.colorDisplay[colorid];
        $scope.colorItems = $scope.filter.colors.length;
        $scope.onToggleFeature();
    }

    /*
    $scope.toggleSilhouette = function(silhouetteid) {
        toggleArray($scope.filter.silhouettes, silhouetteid);

        $scope.silhouetteDisplay[silhouetteid] = !$scope.silhouetteDisplay[silhouetteid];
        $scope.silhouetteItems = $scope.filter.silhouettes.length;
        $scope.onToggleFeature();
    }

    $scope.toggleBeak = function(beakid) {
        toggleArray($scope.filter.beaks, beakid);

        $scope.beakDisplay[beakid] = !$scope.beakDisplay[beakid];
        $scope.beakItems = $scope.filter.beaks.length;
        $scope.onToggleFeature();
    }
    */

    var toggleArray = function(array, value) {
        var index = array.indexOf(value)
        if (index == -1) array.push(value);
        else array.splice(index, 1);
    }
})


/***************************/
/* VogelsDetailsController */
/***************************/
.controller('VogelsDetailsController', function($scope, $rootScope, $sce, $state, $stateParams, $window, $cordovaMedia, $ionicModal, $ionicPopup, DataService, TelFactory, FilterFactory, SettingsFactory) {

    $scope.isPlaying = false;
    SettingsFactory.suppressBackButton(true);


    var id = parseInt($stateParams.id);
    $scope.bird = _.find(DataService.birds, function(bird) {
        return bird.id == id;
    });

    $scope.videoUrl = $sce.trustAsResourceUrl($rootScope.SETTINGS.videoUrl + $scope.bird.video + '&title=' + $scope.bird.bird + '&id=0');

    function getMediaURL(s) {
        if (device.platform.toLowerCase() === "android") return "/android_asset/www/" + s;
        return s;
    }


    $scope.getNumber = function(num) {
        return new Array(num);
    }

    $scope.next = function() {
        $cordovaMedia.pause();
        $state.go('app.vogels-details', {
            id: FilterFactory.nextBirdId(id)});
    }

    $scope.prev = function() {
        $cordovaMedia.pause();
        $state.go('app.vogels-details', {
            id: FilterFactory.prevBirdId(id)});
    }


    $scope.showOverview = function() {
        SettingsFactory.suppressBackButton(false);
        $state.go('app.vogels');
    }



    $scope.toggleSound = function(mp3) {
        if ($scope.isPlaying) {
            $cordovaMedia.pause();
            $scope.isPlaying = false;
        } else {
            $cordovaMedia.play(getMediaURL('media/sounds/') + mp3);
            $scope.isPlaying = true;
        }
    }

    $scope.add = function(bird) {
        var telling = TelFactory.addTellingBirds({
            'id': bird.id,
            'count': 1,
            'bird': bird.bird,
            'reference': bird.reference
        });

        SettingsFactory.suppressBackButton(false);
        $state.go('app.vogeldagboek-add');
    }

    $scope.showConfirmAdd = function(bird) {
        var confirmPopup = $ionicPopup.confirm({
            title: 'Toevoegen aan tellijst',
            template: 'Wil je de ' + bird.bird + ' toevoegen aan de tellijst?',
            cancelText: 'Nee'
        });
        confirmPopup.then(function(res) {
            if (res) {
                $scope.add(bird);
            }
        });
    };

    $ionicModal.fromTemplateUrl('templates/vogels-modal.html', {
        scope: $scope,
        animation: 'slide-in-up'
    }).then(function(modal) {
        $scope.modal = modal;
    });

    $scope.show = function(imageref) {
        $scope.imageref = imageref || $scope.bird.reference;
        $scope.modal.show();
    };

    $scope.close = function() {
        $scope.modal.hide();
    };
});
