angular.module('tuinvogels.controllers', [])

/***************************/
/* AppController           */
/***************************/
.controller('AppCtrl', function($scope, $state, $cordovaDialogs, AuthorizationFactory, StorageFactory, SettingsFactory) {
    
    AuthorizationFactory.getCredentials();

    $scope.$on('menu:back', function(event, status) {
        $scope.suppressBackButton = status;
    });

    $scope.isWeekend = SettingsFactory.isWeekend();
    
    $scope.logout = function() {
        AuthorizationFactory.clearCredentials();
        $state.go('app.home');
    }


    $scope.reset = function() {

        try {
            $cordovaDialogs.confirm("Hiermee verwijdert u het complete Tuinvogeldagboek (alleen in testfase)", "Alles verwijderen", ["Ok", "Annuleer"])
                .then(function(index) {
                    if (index == 1) {
                        StorageFactory.setObject('dagboek', {});
                        $state.go('app.home');
                    }
                });
        } catch(e) {
            $state.go('app.home');
        }
    };

})


/***************************/
/* HomeController          */
/***************************/
.controller('HomeController', function($scope, SettingsFactory, $cordovaGoogleAnalytics) {

    //$cordovaGoogleAnalytics.trackView('Home');
    $scope.showMessage = !SettingsFactory.getWelcomeMessageShown();

    $scope.messageShown = function() {
        $scope.showMessage = false;
        SettingsFactory.setWelcomeMessageShown();
    }
})


/***************************/
/* InfoController          */
/***************************/
.controller('InfoController', function($scope, DataService, $cordovaGoogleAnalytics) {

    try {
        $cordovaGoogleAnalytics.trackView('Over');
    } catch(e) {};

    $scope.records = DataService.info;
})


/***************************/
/* InfoDetailsController   */
/***************************/
.controller('InfoDetailsController', function($scope, $stateParams, DataService) {

    $scope.record = DataService.getLocalInfo($stateParams.id);
})


/***************************/
/* SocialController        */
/***************************/
.controller('SocialController', function($scope, $cordovaGoogleAnalytics, SocialFactory) {

    try {
        $cordovaGoogleAnalytics.trackView('Social');
    } catch(e) {};


    $scope.refresh = function(force) {
        SocialFactory.getTwitterFeedKeywords(force)
            .then(function(results) {
                $scope.feed = results;
                console.log('feed', $scope.feed)
                $scope.$broadcast('scroll.refreshComplete');
            });
    };

    $scope.refresh();
})


/***************************/
/* LoginController         */
/***************************/
.controller('LoginController', function($scope, $rootScope, $state, $ionicPopup, $cordovaGoogleAnalytics, Social, AuthorizationFactory, DataService) {

    try {
        $cordovaGoogleAnalytics.trackView('Login');
    } catch(e) {};

    $scope.model = {};
    $scope.socialUrl = {
        'facebook': $rootScope.SETTINGS.registerUrl + 'facebookapp&cb=' + $rootScope.SETTINGS.oauthcallback,
        'twitter':  $rootScope.SETTINGS.registerUrl + 'twitterapp&cb='  + $rootScope.SETTINGS.oauthcallback
    };
    $scope.intro = DataService.info[3].content;

    $scope.openSocial = function(channel, nextState) {

        nextState = nextState || "app.home";

        Social.login($scope.socialUrl[channel]).then(function(credentials) {
            AuthorizationFactory.setCredentials(credentials);
            $state.go(nextState);
        });
    }

    $scope.login = function() {
        AuthorizationFactory.login($scope.model).then(function(credentials) {

            if (credentials != null) {
                AuthorizationFactory.setCredentials(credentials);
                $state.go("app.home");
            }
            else {
                $ionicPopup.alert({
                    title: 'Ongeldige login',
                    template: 'E-mail adres en/of wachtwoord niet bekend.'
                });
            }
        });
    }

    $scope.passwordMessage = function() {
        $ionicPopup.alert({
            title: 'Wachtwoord vergeten?',
            template: 'U kunt via de website van de <a href="http://www.tuinvogeltelling.nl" external>Nationale Tuinvogeltelling</a> een nieuw wachtwoord aanvragen.'
        });
    }

})


/***************************/
/* ProfielController       */
/***************************/
.controller('ProfielController', function($scope, $state, $ionicPopup, $cordovaGoogleAnalytics, AuthorizationFactory, RegistrationFactory, UserFactory) {

    try {
        $cordovaGoogleAnalytics.trackView('Profiel');
    } catch(e) {};

    var credentials = AuthorizationFactory.getCredentials();
    var registered = (credentials.uuid && credentials.uuid != '');

    $scope.model = {};

    if (registered) {
        RegistrationFactory.get(credentials.uuid).then(function(res) {
            if(res.status != false) {
                $scope.model = res;
                $scope.model['uuid'] = credentials.uuid;
                $scope.model['huisnummer'] = parseInt(res.huisnummer);
                $scope.model['provincie'] = 0;
                $scope.model.bezwaar = (res.bezwaar > 0);
            }
            else {
                var alertPopup = $ionicPopup.alert({
                    title: 'Waarschuwing',
                    template: 'Er is een fout opgetreden. Probeer het (later) nog eens.'
                });
            }
        })
    }
    else {
        RegistrationFactory.getPassword().then(function(data) {
            $scope.model.password = data;
        });
    }



    $scope.register = function() {

        if (!registered) {
            RegistrationFactory.emailExists($scope.model.email).then(function(res) {
                if (!res) {
                    $scope.save();
                }
                else {
                    var alertPopup = $ionicPopup.alert({
                        title: 'Fout',
                        template: 'E-mail adres is al geregistreerd. Log direct in met dit e-mail adres'
                    });
                }
            });
        }
        else {
            $scope.save();
        }
    }

    $scope.save = function() {
        
        RegistrationFactory.save($scope.model).then(function(res) {

            var title, message;

            if (!registered) {
                title = "Bedankt!";
                message = "Uw inschrijving is gelukt. Controleer uw e-mail. Hierin vindt u uw wachtwoord.";
            } else {
                title = 'Opgeslagen';
                message = 'De gegevens zijn opgeslagen.';
            }

            if (res.status == true) {
                UserFactory.setProfile($scope.model);
                
                //automatic login
                AuthorizationFactory.setCredentials({'uuid': res.uuid});
                registered = true;

                var alertPopup = $ionicPopup.alert({
                    title: title,
                    template: message
                });
                alertPopup.then(function(res) {
                    $state.go('app.home');
                });

                
            }
            else {
                var alertPopup = $ionicPopup.alert({
                    title: 'Waarschuwing',
                    template: 'Er is een fout opgetreden. Probeer het (later) nog eens.'
                });
            }
        });
    }
});