angular.module('tuinvogels.services', [])


    .factory('customAuthHttpInterceptor', function ($q) {
        return {
            response: function (response) {
                // do nothing on success
                return response;
            },

            responseError: function (response) {
                var status = response.status;
                if (status == 401) {
                    //AuthorizationFactory.clearCredentials();
                    window.location = "#/app/login";
                    return;
                }
                // otherwise
                return $q.reject(response);
            }
        };
    })

    /*********************/
    /* LoadingFactory    */
    /*********************/
    .factory('LoadingFactory', function ($ionicLoading) {

        return {
            show: function () {
                $ionicLoading.show({
                    template: '<i class="icon ion-refresh"></i>',
                    animation: 'fade-in',
                    noBackdrop: false,
                    delay: 0
                })
            },
            hide: function () {
                $ionicLoading.hide()
            }
        }
    })



    /*********************/
    /* RemoteFactory     */
    /*********************/
    .factory('RemoteFactory', function ($q, $rootScope, $http, $angularCacheFactory, LoadingFactory) {

        return {
            get: function (method, cacheId, forceReload, cacheName, secure) {

                forceReload = forceReload || false;
                cacheName = cacheName || 'defaultCache';
                secure = secure || false;

                var options = {};
                options.method = 'GET';
                options.url = $rootScope.SETTINGS.remoteUrl + method;
                options.timeout = 10000;
                if (secure) {
                    var mycredentials = $rootScope.credentials.uuid;
                    options.headers = {
                        'Authorization': 'Basic ' + btoa(mycredentials)
                    };
                }

                var deferred = $q.defer(),
                    defaultCache = $angularCacheFactory.get(cacheName);

                if (cacheId && defaultCache.get(cacheId) && !forceReload) {
                    deferred.resolve(defaultCache.get(cacheId));
                } else {
                    LoadingFactory.show();
                    $http(options).
                        success(function (data) {
                            if (forceReload) defaultCache.removeAll();
                            if (cacheId) defaultCache.put(cacheId, data);
                            deferred.resolve(data);
                        }).finally(function () {
                            LoadingFactory.hide();
                        });
                }
                return deferred.promise;
            },

            post: function (method, body, secure) {

                secure = secure || false;

                var options = {
                    method: 'POST',
                    url: $rootScope.SETTINGS.remoteUrl + method,
                    data: body,
                    timeout: 10000,
                }

                if (secure && $rootScope.credentials.uuid) {
                    var mycredentials = $rootScope.credentials.uuid;
                    options.headers = {
                        'Authorization': 'Basic ' + btoa(mycredentials)
                    };
                }

                var deferred = $q.defer();

                LoadingFactory.show();

                $http(options).success(function (data) {
                    deferred.resolve(data);
                }).finally(LoadingFactory.hide);

                return deferred.promise;
            },

            removeFromCache: function (cacheId) {
                var defaultCache = $angularCacheFactory.get('defaultCache');
                defaultCache.remove(cacheId);
            }
        }
    })


    /*********************/
    /* SettingsFactory   */
    /*********************/
    .factory('SettingsFactory', function ($window, $rootScope) {

        var welcomeMessage = false,
            suppressBackButton = false;

        return {

            setWelcomeMessageShown: function () {
                welcomeMessage = true;
            },

            getWelcomeMessageShown: function () {
                return welcomeMessage
            },

            suppressBackButton: function (status) {
                suppressBackButton = status;
                $rootScope.$broadcast('menu:back', status);
            },

            isWeekend: function () {
                var now = new Date();
                var startDate = new Date(2019, 0, 25, 8, 0, 0);
                var endDate = new Date(2019, 0, 28, 12, 0);

                return (now > startDate && now < endDate);
            }
        }
    })


    /*********************/
    /* StorageFactory    */
    /*********************/
    .factory('StorageFactory', ['$window',
        function ($window) {
            return {
                set: function (key, value) {
                    $window.localStorage[key] = value;
                },

                get: function (key, defaultValue) {
                    return $window.localStorage[key] || defaultValue;
                },

                setObject: function (key, value) {
                    $window.localStorage[key] = JSON.stringify(value);
                },

                getObject: function (key) {
                    return JSON.parse($window.localStorage[key] || '{}');
                }
            }
        }
    ])



    /*********************/
    /* UserFactory       */
    /*********************/
    .factory('UserFactory', function ($window) {
        return {

            setProfile: function (value) {
                $window.localStorage['profile'] = JSON.stringify(value);
            },

            getProfile: function () {
                return JSON.parse($window.localStorage['profile'] || '{}');
            }
        }
    })


    /*********************/
    /* FilterFactory     */
    /*********************/
    .factory('FilterFactory', function () {

        var currentFilter = {
            size: undefined,
            colors: []
            //beaks: [],
            //silhouettes: []
        };

        var currentBirds = [];

        return {
            setFilter: function (filter) {
                currentFilter = filter;
            },
            getFilter: function () {
                return currentFilter;
            },
            resetFilter: function () {
                currentFilter = {
                    size: undefined,
                    colors: [],
                    beaks: [],
                    silhouettes: []
                };
                return currentFilter;
            },

            determin: function (birds) {

                return _.filter(birds, function (bird) {

                    return _.find(bird.sizes, function (size) {

                        return (currentFilter.size ? size.id == currentFilter.size : true);
                    })

                        &&

                        (function () {
                            for (var i = 0; i < currentFilter.colors.length; i++) {
                                if (_.pluck(bird.colors, 'id').indexOf(currentFilter.colors[i]) < 0) return false
                            }
                            return true;
                        })()

                    /*
                    _.find(bird.colors, function(color) {
                        return (currentFilter.colors.length ? _.indexOf(currentFilter.colors, color.id) > -1 : true);
                    })

                    
                    &&

                    _.find(bird.silhouettes, function(silhouette) {
                        return (currentFilter.silhouettes.length ? _.indexOf(currentFilter.silhouettes, silhouette.id) > -1 : true);
                    })

                    &&

                    _.find(bird.beaks, function(beak) {
                        return (currentFilter.beaks.length ? _.indexOf(currentFilter.beaks, beak.id) > -1 : true);
                    })
                    */
                })
            },

            setCurrentBirds: function (birds) {
                currentBirds = birds;
            },

            getCurrentBirds: function () {
                return currentBirds;
            },

            nextBirdId: function (id) {
                var index = _.indexOf(currentBirds, id);
                if (index == currentBirds.length - 1) return currentBirds[0];
                else return currentBirds[index + 1];
            },

            prevBirdId: function (id) {
                var index = _.indexOf(currentBirds, id);
                if (index == 0) return currentBirds[currentBirds.length - 1];
                else return currentBirds[index - 1];
            }
        }
    })


    /*********************/
    /* TelFactory       */
    /*********************/
    .factory('TelFactory', function () {

        var telling = {
            'id': null,
            'date': new Date(),
            'season': '',
            'uuid': '',
            'preferredimage': '',
            'preferredimageid': null,
            'special': false,
            'remarks': '',
            'birds': []
        },
            activeLijst = null;

        return {
            getTelling: function () {
                return telling;
            },

            setTelling: function (t) {
                telling = t;
            },

            resetTelling: function () {
                telling = {
                    'id': null,
                    'date': new Date(),
                    'season': '',
                    'uuid': '',
                    'preferredimage': '',
                    'preferredimageid': null,
                    'special': false,
                    'remarks': '',
                    'birds': []
                };
            },

            setActiveLijst: function (tellijst) {
                activeLijst = tellijst;
            },

            getActiveLijst: function () {
                return activeLijst;
            },


            initBirdList: function (birds) {

                for (var j = 0; j < birds.length; j++) {

                    var indexInTelling = _.findIndex(telling.birds, { id: birds[j].id });
                    if (indexInTelling > -1) {
                        birds[j].count = telling.birds[indexInTelling].count;
                    } else birds[j].count = 0;
                }

            },


            updateTellingBirds: function (birds) {
                telling.birds = [];

                for (var i = 0; i < birds.length; i++) {
                    if (birds[i].count > 0) {
                        telling.birds.push({
                            'id': birds[i].id,
                            'count': birds[i].count,
                            'bird': birds[i].bird,
                            'reference': birds[i].reference
                        });
                    }
                }
            },


            addTellingBirds: function (data) {
                var index = _.findIndex(telling.birds, { id: data.id });

                if (index > -1) telling.birds[index].count += 1;
                else telling.birds.push(data);
            }

        }
    })


    /*********************/
    /* WeekendFactory    */
    /*********************/
    .factory('WeekendFactory', function (RemoteFactory) {

        var uuid = null;

        return {
            save: function (data) {
                return RemoteFactory.post('weekend.save', { data: data }, true);
            },

            setUUID: function (value) {
                uuid = value;
            },

            getUUID: function () {
                return uuid;
            }
        }
    })


    /*********************/
    /* SyncFactory       */
    /*********************/
    .factory('SyncFactory', function (RemoteFactory) {

        return {
            sync: function (data, uuid) {
                return RemoteFactory.post('sync.sync', { data: data, uuid: uuid }, true);
            }
        }
    })


    /*********************/
    /* SocialFactory     */
    /*********************/
    .factory('SocialFactory', function (RemoteFactory) {

        return {
            getTwitterFeed: function (force) {
                return RemoteFactory.get('social.getTwitterFeed', '/twitter', force);
            },

            getTwitterFeedKeywords: function (force) {
                return RemoteFactory.get('social.getTwitterFeedKeywords', '/twitter', force);
            },

        }
    })


    /************************/
    /* RegistrationFactory  */
    /************************/
    .factory('RegistrationFactory', function (RemoteFactory) {

        return {

            save: function (data) {
                return RemoteFactory.post('register.save', data);
            },

            get: function (uuid) {
                return RemoteFactory.get('register.get', null, null, null, true);
            },

            getPassword: function () {
                return RemoteFactory.get('register.randomPass', null, null, null, true);
            },

            emailExists: function (email) {
                return RemoteFactory.get('register.emailExists&email=' + email, null, null, null, false);
            }
        }
    })

    /************************/
    /* AuthorizationFactory */
    /************************/
    .factory('AuthorizationFactory', function ($window, $rootScope, RemoteFactory) {

        return {

            getCredentials: function () {
                var credentials = $window.localStorage['credentials'];

                if (credentials) {
                    return angular.fromJson(credentials);
                }
                return {
                    uuid: ''
                };
            },

            setCredentials: function (credentials) {
                $rootScope.credentials = credentials;
                $window.localStorage['credentials'] = angular.toJson(credentials);
            },

            clearCredentials: function () {
                $rootScope.credentials = {
                    uuid: ''
                };
                $window.localStorage['credentials'] = angular.toJson($rootScope.credentials);
            },

            login: function (data) {
                return RemoteFactory.post('access.login', data);
            }
        }
    });